#include "explode.h"

#include "game_display.h"
#include "player.h"
#include "options.h"

#include <stdlib.h>
#include <time.h>

void explode_process_detonator(struct xmv_link* detlnk)
{
    #ifndef DEBUG
    struct timespec rpause;
    struct timespec repause;
    #endif
    struct xor_move* detmv=detlnk->xmv;
    xy_t bombx=detmv->to_x;
    xy_t bomby=detmv->to_y;
    xy_t exp_x[4]={0,0,0,0};
    xy_t exp_y[4]={0,0,0,0};
    enum MOVES detdir=detmv->dir;
    su_t i;
    su_t icon;
    #ifndef DEBUG
    xy_t xi;
    #endif
    su_t bomb;
    su_t dead1=0;
    su_t dead2=0;
    bool exp[4]={TRUE,TRUE,TRUE,TRUE};
    #ifndef DEBUG
    rpause.tv_sec=0;
    if(player.replay)
        rpause.tv_nsec=options_replay_speed(options->replay_speed);
    else
        rpause.tv_nsec=60000000L;
    #else
    printf("explode_process_detonator(detlnk=%lx)",(unsigned long)detlnk);
    printf("detlnk->xmv=%lx\n",(unsigned long)detlnk->xmv);
    #endif
    exp_x[0]=detmv->from_x;
    exp_y[0]=detmv->from_y;
    exp_x[1]=bombx;
    exp_y[1]=bomby;
    switch((bomb=map->buf[bomby][bombx])){
        case ICON_H_BOMB:
            exp_x[2]=bombx-1;
            exp_y[2]=bomby;
            if(detdir==MV_LEFT)
                exp[3]=FALSE;
            else{
                exp_x[3]=bombx+1;
                exp_y[3]=bomby;
            }
            break;
        case ICON_V_BOMB:
            if(detdir==MV_DOWN)
                exp[2]=FALSE;
            else{
                exp_x[2]=bombx;
                exp_y[2]=bomby-1;
            }
            exp_x[3]=bombx;
            exp_y[3]=bomby+1;
            break;
    }
    for(i=0;i<4;i++)
        if(exp_x[i]==0||exp_x[i]>MAP_W-2
         ||exp_y[i]==0||exp_y[i]>MAP_H-2)
            exp[i]=FALSE;
        #ifdef DEBUG
        else
            printf("valid explosion point [%d] x:%d y%d\n",i,exp_x[i],exp_y[i]);
        #endif
    #ifndef DEBUG
    for(xi=ICON_EXPLOSION1;xi<=ICON_EXPLOSION3;xi++){
        for(i=0;i<4;i++){
            if(exp[i]){
                game_win_icon_display(exp_x[i],exp_y[i],xi);
            }
        }
        wrefresh(game_win);
        rpause.tv_nsec/=1.175;
        nanosleep(&rpause,&repause);
    }
    #endif
    for(i=0;i<4;i++)
        if(exp[i]){
            #ifndef DEBUG
            game_win_icon_display(exp_x[i],exp_y[i],ICON_SPACE);
            #endif
            icon=map->buf[exp_y[i]][exp_x[i]];
            if(actions[icon].mvini==MVI_PLAYER){
                if(dead1==0)
                    dead1=icon;
                else
                    dead2=icon;
            }
            else if(icon==ICON_SWITCH){
                player.wall_vis=(player.wall_vis?FALSE:TRUE);
                #ifndef DEBUG
                init_wall(map->level,player.wall_vis);
                #endif
            }
            map->buf[exp_y[i]][exp_x[i]]=ICON_SPACE;
            if(icon==ICON_SWITCH)
                /* so exploded switch disappears */
                game_win_display();
        }
    #ifndef DEBUG
    wrefresh(game_win);
    #endif
    /*  if detonator part of chain, remove it from chain, delete it,
        and set detlnk to point to new head of chain. if not part
        of a chain, unlink, and delete.
    */
    if(dead1)
        player_death(dead1);
    if(dead2)
        player_death(dead2);
    if(detmv->right){
        #ifdef DEBUG
        printf("pruning chain: detlink->xmv=(was)%lx (now %lx->right)= %lx\n",
            (unsigned long)detlnk->xmv,(unsigned long)detmv,
            (unsigned long)detmv->right);
        printf("freeing detmv");
        #endif
        detlnk->xmv=detmv->right;
        free(detmv);
        detmv=0;
    }
    else if(detmv->up){
        #ifdef DEBUG
        printf("pruning chain: detlink->xmv=(was)%lx (now %lx->up)= %lx\n",
            (unsigned long)detlnk->xmv,(unsigned long)detmv,
            (unsigned long)detmv->up);
        printf("freeing detmv");
        #endif
        detlnk->xmv=detmv->up;
        free(detmv);
        detmv=0;
    }
    else
        free(xmvlist_unlink_xor_move(detlnk));
    if(bomb==ICON_H_BOMB)
        explode_process_h_blast(bombx,bomby,detdir);
    else
        explode_process_v_blast(bombx,bomby,detdir);
}

void explode_process_h_blast(xy_t bombx, xy_t bomby, su_t detdir)
{
    struct xor_move* xmv;
    xy_t chkx[3];
    xy_t chky[3];
    xy_t dir[3];
    su_t i;
    chkx[0]=bombx+1;
    chky[0]=bomby-1;
    dir[0]=MV_DOWN;
    if(detdir==MV_DOWN){
        chkx[1]=bombx+2;
        chky[1]=bomby;
        dir[1]=MV_LEFT;
    }
    else{
        chkx[1]=bombx;
        chky[1]=bomby-1;
        dir[1]=MV_DOWN;
    }
    chkx[2]=bombx-1;
    chky[2]=bomby-1;
    dir[2]=MV_DOWN;
    for(i=0;i<3;i++)
        if(chkx[i]>0&&chkx[i]<MAP_W-1&&chky[i]>0&&chky[i]<MAP_H-1)
            if(!xmvlist_contains_coord(chkx[i],chky[i],0,0))
                if((xmv=create_gravity_chain_xydir(chkx[i],chky[i],dir[i])))
                    xmvlist_append_xor_move(xmv);
}

void explode_process_v_blast(xy_t bombx, xy_t bomby, su_t detdir)
{
    struct xor_move* xmv;
    xy_t chkx[3];
    xy_t chky[3];
    su_t dir[3];
    su_t i;
    chkx[0]=bombx+1;
    chky[0]=bomby-1;
    dir[0]=MV_LEFT;
    if(detdir==MV_LEFT){
        chkx[1]=bombx;
        chky[1]=bomby-2;
        dir[1]=MV_DOWN;
    }
    else{
        chkx[1]=bombx+1;
        chky[1]=bomby;
        dir[1]=MV_LEFT;
    }
    chkx[2]=bombx+1;
    chky[2]=bomby+1;
    dir[2]=MV_LEFT;
    for(i=0;i<3;i++)
        if(chkx[i]>0&&chkx[i]<MAP_W-1&&chky[i]>0&&chky[i]<MAP_H-1)
            if(!xmvlist_contains_coord(chkx[i],chky[i],0,0))
                if((xmv=create_gravity_chain_xydir(chkx[i],chky[i],dir[i])))
                    xmvlist_append_xor_move(xmv);
}
