#include "play_xor.h"
#include <stdlib.h>
#include <time.h>

#ifndef DEBUG
#include <curses.h>
#else
#include <stdio.h>
#endif

#include "options.h"
#include "scores.h"

bool play_xor(lvl_t level, bool from_start)
{
    int key=0;
    ctr_t mv;
    su_t move;
    su_t state=PLAY_CONTINUE;
    player.replay=0;
    if(from_start){
        #ifndef DEBUG
        replay.level=level;
        for(mv=MAX_MOVES;mv>0;mv--)
            replay.moves[mv]=0;
        init_wall(level,TRUE);
        #endif
        xor_map_create();
        xor_map_load(level);
        player_init();
    }
    #ifndef DEBUG
    else
        level=replay.level;
    #endif
    game_win_display();
    #ifndef DEBUG
    wclear(info_win);
    wattrset(info_win,COLOR_PAIR(0));
    box(info_win,0,0);
    info_win_update_player_icon();
    info_win_update_map(player.have_map);
    #endif
    info_win_display();
    while(state==PLAY_CONTINUE||state==PLAY_PROCESS_MOVE){
        #ifndef DEBUG
        if((key=wgetch(game_win))!=ERR){
        #else
        printf("\nz-left x-right '-up /-down p-player q-quit ???");
        key=getchar();
        printf("\n");
        #endif
            move=MV_NONE;
            switch(key){
                #ifndef DEBUG
                case KEY_LEFT:
                #endif
                case 'z':   move=MV_LEFT;   break;
                #ifndef DEBUG
                case KEY_RIGHT:
                #endif
                case 'x':   move=MV_RIGHT;  break;
                #ifndef DEBUG
                case KEY_UP:
                #endif
                case '\'':  move=MV_UP;     break;
                #ifndef DEBUG
                case KEY_DOWN:
                #endif
                case '/':   move=MV_DOWN;   break;
                #ifndef DEBUG
                case '\r':
                #endif
                case 'p':   move=MV_PLAYER_SWAP;        break;
                case 'q':
                case 'Q':   move=MV_PLAYER_QUIT;        break;
                #ifndef DEBUG
                case 'm':   game_win_map_display();     break;
                case '1':   options->scroll_thresh=1;    break;
                case '2':   options->scroll_thresh=2;    break;
                case '3':   options->scroll_thresh=3;    break;
                #endif
            }
            if(move){
                if((state=player_move(move))&PLAY_RECORD){
                    #ifndef DEBUG
                    replay.moves[player.moves_remaining--]=move;
                    #endif
                    state^=PLAY_RECORD;
                }
                info_win_display();
                if((!player.p0_alive)&&(!player.p1_alive))
                    state=PLAY_GOTCHA;
            }
        #ifndef DEBUG
        }
        #else
        game_win_display();
        #endif
    }
    if(state==PLAY_COMPLETE)
        save_score(map->level,MAX_MOVES-player.moves_remaining);
    #ifndef DEBUG
    if(player.moves_remaining<MAX_MOVES)
        return replay_xor(TRUE);
    #endif
    return FALSE;
}

#ifndef DEBUG
bool replay_xor(bool ask)
{
    int key;
    su_t state;
    struct timespec rpause;
    struct timespec repause;
    bool continue_play=FALSE;
    bool ask_continue=player.p0_alive|player.p1_alive;
    rpause.tv_sec=0;
    rpause.tv_nsec=options_replay_speed(options->replay_speed);
    do{
        if(ask==TRUE){
            wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
            mvwprintw(game_win,11,8,"                   ");
            if(ask_continue)
                mvwprintw(game_win,12,8," Replay (Y/N/C/S)? ");
            else
                mvwprintw(game_win,12,8,"  Replay (Y/N/S)?  ");
            mvwprintw(game_win,13,8,"                   ");
            key=wgetch(game_win);
            switch(key){
                case 'n': case 'N':
                case 'q': case 'Q':
                    xor_map_destroy();
                    return FALSE;
                case 's': case 'S':
                    replay_save();
                    xor_map_destroy();
                    return FALSE;
                case 'c': case 'C':
                    if(ask_continue)
                        if(player.p0_alive||player.p1_alive)
                            continue_play=TRUE;
            }
        }
        else{
            ask=TRUE;
            key='y';
        }
        if(key=='y'||key=='Y'){
            xor_map_create();
            xor_map_load(replay.level);
            player_init();
            player.replay=TRUE;
            init_wall(replay.level,TRUE);
            game_win_display();
            #ifndef DEBUG
            wclear(info_win);
            wattrset(info_win,COLOR_PAIR(0));
            box(info_win,0,0);
            info_win_update_map(player.have_map);
            #endif
            info_win_display();
            info_win_update_player_icon();
            nodelay(game_win,TRUE);
            state=PLAY_CONTINUE;
            while(state==PLAY_CONTINUE
                ||state==PLAY_PROCESS_MOVE)
            {
                nanosleep(&rpause,&repause);
                if((key=wgetch(game_win))!=ERR){
                    if(key=='q'||key=='Q')
                        state=player_move(MV_PLAYER_QUIT);
                    else if(key>='1'&&key<='9')
                        rpause.tv_nsec=options_replay_speed(key);
                }
                else
                    state=player_move(
                        replay.moves[player.moves_remaining--])
                            ^PLAY_RECORD;
                info_win_display();
                if((!player.p0_alive)&&(!player.p1_alive))
                    state=PLAY_GOTCHA;
            }
            nodelay(game_win,FALSE);
            if(state==PLAY_GOTCHA)
                ask_continue=FALSE;
            else
                ask_continue=TRUE;
        }
    }while(!continue_play);
    state=PLAY_CONTINUE;
    if(replay.moves[player.moves_remaining+1]==MV_PLAYER_QUIT)
        player.moves_remaining++;
    return TRUE;
}

void replay_save()
{
    char buf[32];
    FILE* fp;
    int key;
    echo();
    wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
    mvwprintw(game_win,11,0,"                                        ");
    mvwprintw(game_win,12,0," Filename:                              ");
    mvwprintw(game_win,13,0,"                                        ");
    wmove(game_win,12,10);
    wgetnstr(game_win,buf,30);
    noecho();
    if((fp=fopen(buf,"r"))){
        mvwprintw(game_win,15,0,"                          ");
        mvwprintw(game_win,16,0," Exists! Overwrite (y/n)? ");
        mvwprintw(game_win,17,0,"                          ");
        key=wgetch(game_win);
        if(key=='n'||key=='N')
            return;
        fclose(fp);
    }
    if(!(fp=fopen(buf,"w"))){
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return;
    }
    if(!fwrite(&replay,sizeof(struct xor_replay),1,fp))
    {
        fclose(fp);
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return;
    }
    fclose(fp);
    mvwprintw(game_win,18,0,"        ");
    mvwprintw(game_win,19,0," Saved! ");
    mvwprintw(game_win,20,0,"        ");
    wgetch(game_win);
}


lvl_t replay_load()
{
    char buf[32];
    FILE* fp;
    echo();
    wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
    mvwprintw(game_win,11,0,"                                        ");
    mvwprintw(game_win,12,0," Filename:                              ");
    mvwprintw(game_win,13,0,"                                        ");
    wmove(game_win,12,10);
    wgetnstr(game_win,buf,30);
    noecho();
    if(!(fp=fopen(buf,"r"))){
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return -1;
    }
    if(!fread(&replay,sizeof(struct xor_replay),1,fp))
    {
        fclose(fp);
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return -1;
    }
    mvwprintw(game_win,18,0,"         ");
    mvwprintw(game_win,19,0," Loaded! ");
    mvwprintw(game_win,20,0,"         ");
    if(replay_xor(FALSE))
        return 100;
    return replay.level;
}

#endif
