#include <stdlib.h>

#include "level_menu.h"

#ifndef DEBUG
#include <curses.h>
#else
#include "movelist.h"
#endif

#include "play_xor.h"
#include "options.h"
#include "scores.h"

#ifndef DEBUG
void init_curses();
void splash();
#endif

int main(void)
{
    lvl_t lastlevel;
    lvl_t level=1;
    if(!options_create()){
        printf("Please either install XorCurses by running 'sudo make install' within the\n");
        printf("XorCurses-0.0.6 source directory, or always start XorCurses from within that\n");
        printf("directory.\n");
        exit(1);
    }
    if(!options_create_map_names()){
        printf("A problem occurred establishing the map names, sorry :-(\n");
        exit(1);
    }
    create_scores();
    load_scores();
    #ifndef DEBUG
    init_curses();
    game_win_create();
    info_win_create();
    init_icons();
    do{
        init_pair(ICON_WALL,COLOR_WHITE,COLOR_RED);
        splash();
        lastlevel=level;
        level=level_menu(level);
        switch(level){
            case 100:
                if((level=replay_load())<0)
                    level=lastlevel;
                else if(level==100){
                    while(play_xor(level,FALSE));
                        level=replay.level;
                    level=replay.level;
                }
                wclear(info_win);
                wrefresh(info_win);
                break;
            case 999:
                level=lastlevel;
                help();
                break;
            default:
                if(level>=0&&level<=15){
                    if(play_xor(level,TRUE)){
                        while(play_xor(level,FALSE));
                        level=replay.level;
                    }
                    wclear(info_win);
                    wrefresh(info_win);
                }
        }
    }while(level>=0);
    #else
    printf("struct sizes:\n\txor_move %ld\n\txmv_link %ld\n\txmv_list %ld\n",
        sizeof(struct xor_move), sizeof(struct xmv_link), sizeof(struct xmv_list));
    do{
        if((level=level_menu(level))>=0){
            play_xor(level,TRUE);
        }
    }while(level>=0);
    #endif
    #ifndef DEBUG
    game_win_destroy();
    info_win_destroy();
    endwin();
    #endif
    destroy_scores();
    options_destroy_map_names();
    options_destroy();
    return 0;
}

#ifndef DEBUG
void init_curses()
{
    initscr();
    if(has_colors()==FALSE){
        endwin();
        printf("Your terminal does not support color\n");
        exit(1);
    }
    noecho();
    nonl();
    cbreak();
    nodelay(game_win,FALSE);
    start_color();
    curs_set(0);
}

void splash()
{
    NCURSES_COLOR_T mc_bg=COLOR_BLUE;
    NCURSES_COLOR_T mc_fg=COLOR_BLACK;
    char* xortitle[]={
        "--  --  ---  ---\\",
        " \\  /  /   \\ |   |",
        "  \\/   |   | |--/",
        "  /\\   |   | |  \\",
        " /  \\  \\   / |   \\",
        "--  --  ---  - Curses",
        "X"
    };
    char* xormask[]={
        " ___*******___",
        "/   -------   \\",
        "|             |",
        "\\  ---\\ /---  /",
        " | \\--- ---/ |",
        "/             \\",
        "|             |",
        "\\  \\\\     //  /",
        " \\  \\\\   //  /",
        "  \\  \\\\ //  /",
        "   \\  \\_/  /",
        "    \\     /",
        "     \\___/",
        "X"
    };
    char* xorlns[]={
        "           ---           ---",
        "         ------         ------",
        "      ----------       ----------",
        "",
        " -------------------------------------",
        "  XorCurses-0.0.6b     by jwm-art.net",
        "X"
    };
    char** ptr1;
    char* ptr2;
    xy_t x;
    xy_t y;
    bool msk=FALSE;
    wclear(game_win);
    init_pair(ICON_XXX,mc_fg,mc_bg);
    ptr1=xortitle;
    ptr2=ptr1[0];
    y=0;
    while(*ptr1[0]!='X'){
        ptr2=*ptr1;
        x=10;
        while(*ptr2){
            if(*ptr2!=' '){
                wattrset(game_win,COLOR_PAIR(ICON_WALL));
            }
            else
                wattrset(game_win,COLOR_PAIR(0));
            mvwaddch(game_win,y,x,*ptr2);
            x++;
            ptr2++;
        }
        ptr1++;
        y++;
    }
    ptr1=xormask;
    ptr2=ptr1[0];
    y=7;
    wattrset(game_win,COLOR_PAIR(0));
    while(*ptr1[0]!='X'){
        ptr2=*ptr1;
        x=12;
        msk=FALSE;
        while(*ptr2){
            if(*ptr2!=' '){
                wattrset(game_win,COLOR_PAIR(ICON_SWITCH));
                msk=TRUE;
            }
            else if(msk==TRUE)
                wattrset(game_win,COLOR_PAIR(ICON_XXX));
            if(*ptr2!='*')
                mvwaddch(game_win,y,x,*ptr2);
            x++;
            ptr2++;
        }
        ptr1++;
        y++;
    }
    ptr1=xorlns;
    ptr2=ptr1[0];
    y=16;
    wattrset(game_win,COLOR_PAIR(0));
    while(*ptr1[0]!='X'){
        ptr2=*ptr1;
        x=0;
        while(*ptr2){
            if(*ptr2!=' ')
                mvwaddch(game_win,y,x,*ptr2);
            x++;
            ptr2++;
        }
        ptr1++;
        y++;
    }
    wrefresh(game_win);
}

#endif

